-hmm ../../../model/hmm/hub4_cd_continuous_8gau_1s_c_d_dd
-fdict ../../../model/lm/an4/filler.dict 
-dict ../../../model/lm/an4/an4.dict 
-input_endian little
-upperf	6855.49756
-lowerf	133.33334
-nfilt	40
-nfft 512
-wlen 0.025625
-samprate 16000
-lmctlfn ../../../model/lm/an4/an4.ug.cls.lmctl
-ctl_lm  ../../../model/lm/an4/an4.ctl_lm
-ctl ../../../model/lm/an4/an4.ctl 
-agc none 
-varnorm no 
-cmn prior 
-cmninit 12.0
-subvqbeam 1e-02 
-epl 4 
-fillprob 0.02 
-feat 1s_c_d_dd 
-lw 9.5 
-maxwpf 1 
-beam 1e-40 
-pbeam 1e-30 
-wbeam 1e-20 
-maxhmmpf 1500 
-wend_beam 1e-1 
-ci_pbeam 1e-3
-ds 4
-tighten_factor 0.30
-phypdump 0
